"""Contains the main logic of Janch
"""

import asyncio

from janch.utils import context
from janch.utils.constants import NO_ERROR

_state = {
    'is_header_logged': False
}


def debug(message):
    """Temporarily placed for enabling and disabling debugging

    Will be replaced by logging library later
    """

    (f"*** {message} *** ")


async def inspect(gathered, settings):
    """Runs inspection on gathered data. Loads the inspector based on the settings

    Args:
        gathered: dict
        settings: dict with settings from the inspect section of the config yml

    Returns: dict containing results of the inspection

    """
    debug("Inspecting")

    ret = {}

    DEFAULT = 'regex'

    for field_name, details in settings.items():

        if field_name in gathered:

            if isinstance(details, dict):
                type = details.get('type') or DEFAULT
                value = details.get('value')
            elif not isinstance(details, str):
                type = 'equals'
                value = details
            else:
                type = DEFAULT
                value = details

            inspector = context.inspectors[type](None)

            inspected = await inspector.inspect(gathered[field_name], value)
            ret[field_name] = inspected
        else:
            ret[field_name] = None

    debug("Inspection Completed")

    return ret


async def gather(settings):
    """Use the settings to gather information. Use the settings type to use the
    right gatherer class

    Args:
        settings: dict representing gather related settings from the config yml

    Returns: dict gathered data

    """
    debug("Gathering")
    type = settings.get("type") or "http"
    gatherer = context.gatherers.get(type)

    gathered = None

    if gatherer:
        gathered = await gatherer(settings).gather()

    debug("Gathering Completed")

    return gathered


async def furnish(item, settings, gathered, inspected):
    """Load the formatter to format the results of gathering and inspection

    Args:
        item: str The name of the item from the yml
        settings: dict The full settings
        gathered: dict The gathered information
        inspected: dict The inspected information

    Returns: str blob that be printed or logged

    """
    debug("Formatting")
    type = 'simple'
    formatter = context.formatters[type]

    if formatter:
        instance = formatter()
        formatted = await instance.format(item, settings, gathered, inspected)

        return formatted

    debug("Formatting Completed")


async def log(formatted):
    """Log the formatted data

    Args:
        formatted: str

    Returns: bool

    """

    debug("Logging")
    type = 'cli'

    logger = context.loggers[type]

    if logger:
        method = logger().log

        logged = await method(formatted)

        return logged

    debug("Logging completed")


def _set_up_default_item_settings(settings):
    if not 'inspect' in settings:
        settings.update({'inspect': {}})

    if not 'error' in settings.get('inspect'):
        settings['inspect'].update({'error': NO_ERROR})


async def start_item(item, settings):
    """Runs Janch process on a specific item

    Args:
        item: str name of the item
        settings: dict configuration of the item

    Returns:

    """
    debug(f"Starting {item}")

    _set_up_default_item_settings(settings)

    gathered = await gather(settings['gather'])
    inspected = await inspect(gathered, settings.get('inspect', {}))
    header, body = await furnish(item, settings, gathered, inspected)

    if not _state.get('is_header_logged'):
        header_logged = await log(header)
        _state.update({'is_header_logged': True})

    logged = await log(body)

    debug("Complete")


async def start():
    """Called to start the Janch process

    Returns:

    """
    config = context.config
    await asyncio.gather(*(start_item(item, settings) for item, settings in config.items()))



"""Contains dictionaries representing all the settings

The dictionaries are populated during initialization
"""

config = {}
gatherers = {}
inspectors = {}
loggers = {}
formatters = {}
environment = {}



"""Contains different types of loggers
"""
from abc import ABC


class Logger(ABC):
    """The abstract base class for Logger

    """

    @staticmethod
    def type() -> str:
        """Override to return string type of logger

        Returns: str

        """
        raise NotImplementedError("Return a type string that acts as the name for the logger")

    async def main(self, message) -> bool:
        """Override to implement the logging

        Args:
            message: str the message that needs to be logged

        Returns: bool whether logging worked

        """
        raise NotImplementedError("Implement this method to log the message")

    async def log(self, message):
        """Calls the main method.

        Will be used to reformat parameters for main

        Args:
            message: str

        Returns: bool

        """
        return await self.main(message)


class CLILogger(Logger):
    """Logs to command line"""

    @staticmethod
    def type() -> str:
        """Returns 'cli' as the name of this logger

        Returns: str

        """
        return 'cli'

    async def main(self, message):
        """Simply prints the message

        Args:
            message: str Message to be logged

        Returns: bool Currently always returns True

        """
        print(message)

        return True


def get_default_loggers():
    """Returns all loggers with the logger type/id as key

    Returns:

    """
    loggers = [CLILogger]

    return {l.type(): l for l in loggers}



import click
import yaml
from dotenv.main import dotenv_values

from janch.api.info import *
from janch.api.main import start, init
from janch.utils.display import FixedWidth

dotenv_path = '.env'


@click.group(invoke_without_command=False)
def main():
    "Janch is a system checker configured using YAML"


@click.argument('file', type=click.File('r'))
@click.option('--item', type=str, required=False, help="Name of the item key from the yml file")
@main.command()
def run(file, item):
    """Run janch using a defined yaml file"""
    context_dict = yaml.load(file.read(), Loader=yaml.SafeLoader)

    # import json
    # print(json.dumps(context_dict))

    if item:
        selected_item = {item: context_dict.get(item)} if item in context_dict else None
    else:
        selected_item = context_dict

    if selected_item:
        init(selected_item
             , dotenv_values(dotenv_path))
        start()
    else:
        click.echo(f"Item {item} not found")
        exit()


@main.group(invoke_without_command=False)
def utils():
    "Show info about some utilities"


def _utils_info_prep(things_with_docs: dict):
    columns = {
        'type': 16,
        'description': 64
    }

    display = FixedWidth(columns)

    for gType, gClass in things_with_docs.items():
        display.add_row({"type": gType, "description": gClass.__doc__.split('\n')[0]})

    return display.get_header(), display.format()


@utils.command()
def gatherers():
    "Show information about all the gatherers"
    columns = {
        'type': 16,
        'description': 64,
        'params': 32,
        'output': 32
    }

    display = FixedWidth(columns)

    for gType, gClass in get_all_gatherers().items():
        display.add_row({
            "type": gType,
            "description": gClass.__doc__.split('\n')[0],
            "params": ','.join(gClass.get_input_fields()),
            "output": ','.join(gClass.get_output_fields())
        })

    click.echo(display.get_header())
    click.echo(display.format())


@utils.command()
def inspectors():
    "Show information about all the inspectors"
    type_and_class = get_all_inspectors()
    header, body = _utils_info_prep(type_and_class)
    click.echo(header)
    click.echo(body)


@utils.command()
def loggers():
    "Show information about all the loggers"
    type_and_class = get_all_loggers()
    header, body = _utils_info_prep(type_and_class)
    click.echo(header)
    click.echo(body)


@utils.command()
def formatters():
    """Show information about all the formatters"""
    type_and_class = get_all_formatters()
    header, body = _utils_info_prep(type_and_class)
    click.echo(header)
    click.echo(body)



"""Gatherers are classes that collect the data from various sources
"""
import asyncio
from abc import ABC

import aiohttp

from janch.utils.constants import NO_ERROR


class Gatherer(ABC):
    """Abstract base class for all Gatherer classes
    """

    def __init__(self, settings):
        self.settings = settings

    @staticmethod
    def get_input_fields() -> list:
        """The list of fields needed for this gatherer to work

        Returns: list

        """
        raise NotImplementedError("Should be a string array of field names")

    @staticmethod
    def get_output_fields() -> list:
        """The list of fields that will be output by this gatherer

        Returns:

        """
        raise NotImplementedError("Should return an array of output fields")

    @staticmethod
    def type():
        """The gatherer type

        Returns:

        """
        raise NotImplementedError("Return name of gatherer")

    async def main(self, *args):
        """The main method that should contain the gatherer logic

        Args:
            *args: fields that should be the same as the ones in get_input_fields method

        Returns:

        """
        raise NotImplementedError("Should use the args to gather")

    async def gather(self):
        """This method prepares the parameters for the main method

        This method need not be overridden

        Returns: dict

        """
        params = [self.settings[p] for p in self.get_input_fields()]
        output = self.get_output_fields()
        method = self.main

        gathered = await method(*params)

        for k, v in gathered.items():
            assert k in output

        if not gathered.get('error'):
            gathered['error'] = NO_ERROR

        return gathered


class HttpGatherer(Gatherer):
    """Gather information from a http(s) source

    """

    @staticmethod
    def type():
        """The HttpGatherer type

        Returns: str http

        """
        return "http"

    @staticmethod
    def get_input_fields():
        """The input fields ['url']

        Returns: list

        """
        return ['url']

    @staticmethod
    def get_output_fields():
        """The output fields ['status', 'headers', 'html', 'error']

        Returns: list

        """
        return ['status', 'headers', 'html', 'error']

    async def main(self, url):
        """Makes a request to the url

        Args:
            url: str The http(s) which needs to be requested

        Returns: dict

        """
        import sys

        ret = {'error': None}

        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url, ssl=False) as response:
                    ret['status'] = response.status
                    ret['headers'] = str(response.headers)
                    ret['html'] = str(await response.text())
        except Exception as e:
            ex_type, ex_value, ex_traceback = sys.exc_info()

            ret['error'] = ex_value

        return ret


class CommandGatherer(Gatherer):
    """Execute a shell command

    """

    @staticmethod
    def type():
        """The CommandGatherer type

        Returns: str command

        """
        return "command"

    @staticmethod
    async def run_command(shell=False, *args):
        """
        Example from the following were modified
        https://asyncio.readthedocs.io/en/latest/subprocess.html
        https://docs.python.org/3/library/asyncio-subprocess.html

        :param args:
        :return:
        """

        # Create subprocess
        if shell:
            process = await asyncio.create_subprocess_shell(
                args[0],
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE)
        else:
            process = await asyncio.create_subprocess_exec(
                *args,
                # stdout must a pipe to be accessible as process.stdout
                stdout=asyncio.subprocess.PIPE)
        # Wait for the subprocess to finish
        stdout, stderr = await process.communicate()
        # Return stdout
        return stdout.decode().strip(), stderr.decode().strip() if stderr else None

    @staticmethod
    def get_input_fields():
        """The input fields ['command_str']

        Returns: list

        """
        return ['command_str']

    @staticmethod
    def get_output_fields():
        """The output fields ['result', 'error']

        Returns: list

        """
        return ['result', 'error']

    async def main(self, command_str):
        """Executes the command

        Args:
            command_str: str the shell command to be executed

        Returns: dict

        """
        output, error = await CommandGatherer.run_command(True, *[command_str])

        return {
            'result': output,
            'error': error
        }


class GrepGatherer(Gatherer):
    """Execute a grep command

    """

    @staticmethod
    def type():
        """The GrepGatherer type

        Returns: str grep

        """
        return "grep"

    @staticmethod
    def get_input_fields():
        """The input fields ['filepath', 'search']

        Returns: list

        """
        return ['filepath', 'search']

    @staticmethod
    def get_output_fields():
        """The output fields ['result', 'line_count', 'error']

        Returns: list

        """
        return ['result', 'line_count', 'error']

    async def main(self, filepath, search):
        """Greps the file in the filepath with the search string

        Args:
            filepath: str
            search: str

        Returns: dict

        """
        output, error = await CommandGatherer.run_command(False, *['grep', search, filepath])

        ret = {
            'result': output,
            'line_count': output.count('\n') + 1,
            'error': error
        }

        return ret


def get_default_gatherers():
    """Returns all the gatherers as dict

    Returns: dict[str: Gatherer]

    """
    gatherers = [HttpGatherer, GrepGatherer, CommandGatherer]

    return {c.type(): c for c in gatherers}



"""This modules contains code that enable the components of the Janch config file

"""
from janch.components.formatters import get_default_formatters
from janch.components.gatherers import get_default_gatherers
from janch.components.inspectors import get_default_inspectors
from janch.components.loggers import get_default_loggers

__all__ = [
    'get_default_gatherers',
    'get_default_inspectors',
    'get_default_formatters',
    'get_default_loggers']



"""The main module for pieces of Janch
"""
import asyncio
from typing import Dict

from janch.components import *
from janch.components.formatters import Formatter
from janch.components.gatherers import Gatherer
from janch.components.inspectors import Inspector
from janch.components.loggers import Logger
from janch.utils import *


def init(config: dict, environment: Dict[str, str],
         gatherers: Dict[str, Gatherer] = None,
         inspectors: Dict[str, Inspector] = None,
         formatters: Dict[str, Formatter] = None,
         loggers: Dict[str, Logger] = None):
    """Initialize Janch by passing before using it programmatically

    Args:
        config: dict representation of the Janch yml
        environment: dict representing environment variables from dotenv
        gatherers: dict of the form {str : Gatherer}
        inspectors: dict of the form {str : Inspector}
        formatters: dict of the form {str : Formatter}
        loggers: dict of the form {str : Logger}

    Returns:

    """
    context.gatherers.update(get_default_gatherers())
    context.inspectors.update(get_default_inspectors())
    context.formatters.update(get_default_formatters())
    context.loggers.update(get_default_loggers())

    context.config.update(config)
    context.gatherers.update(gatherers or {})
    context.inspectors.update(inspectors or {})
    context.loggers.update(loggers or {})
    context.formatters.update(formatters or {})
    context.environment.update(environment)


def update_gatherers(gatherer: Dict[str, Gatherer]):
    """Replace or add a Gatherer. The key should be gatherer type str

    Args:
        gatherer: dict

    Returns:

    """
    context.gatherers.update(gatherer)


def update_inspectors(inspector: Dict[str, Inspector]):
    """Replace or add a Inspector. The key should be inspector type

    Args:
        inspector: dict

    Returns:

    """
    context.inspectors.update(inspector)


def update_loggers(logger: Dict[str, Logger]):
    """Replace or add a Logger. The key should be logger type

    Args:
        logger: dict

    Returns:

    """
    context.loggers.update(logger)


def update_formatters(formatter: Dict[str, Formatter]):
    """Replace or add a Formatter. The key should be formatter type

    Args:
        formatter: dict

    Returns:

    """
    context.formatters.update(formatter)


def start():
    """Start the Janch after it has been initialized or with default settings
    Returns:

    """
    try:
        asyncio.run(engine.start())
    except Exception as e:
        raise e



"""Contains the utilities to display things on the screen

"""


class FixedWidth():
    """Utility to display information in a tabulated manner

    """

    def __init__(self, dimensions: dict):
        """

        Args:
            dimensions: dict containing fields as keys and column width (ints) as values
        """
        self.dim = dimensions
        self.rows = []
        self.delimiter = ''

    def get_header(self):
        """Returns the header row in a tabulated manner

        Returns: str

        """
        ret = ''
        for k, v in self.dim.items():
            ret += FixedWidth.fit(v, str(k))
            ret += self.delimiter

        return ret

    def add_row(self, row: dict):
        """Add row to the collection that is to be displayed

        Args:
            row: dict with fields as keys and values as values

        Returns:

        """
        self.rows.append(row)

    def set_delimiter(self, character):
        """Set delimiter such as '|'

        Args:
            character: str

        Returns:

        """
        self.delimiter = character

    @staticmethod
    def fit(size: int, txt: str):
        """Forces a txt to fit into the required size.

        Long texts get truncated and appended with a '..'

        Args:
            size: int the size of to fit the text in
            txt: str the text that needs to be resized

        Returns:

        """
        # Length
        l = len(txt)

        # dimension of field
        d = size

        # number of spaces to append
        s = d - l if l <= d else 0

        # ellipsis
        e = '..' if l > d else ''

        return txt[0:(l if l <= d else (d - len(e)))] + e + ' ' * s

    def format(self):
        """Iterates the rows and formats them.

        Returns: str is a text blob that can be printed

        """
        ret = ''
        for row in self.rows:
            ret += self.delimiter
            for k, v in self.dim.items():
                ret += FixedWidth.fit(v, str(row[k]))
                ret += self.delimiter

            ret += '\n'

        # The following removes trailing newline
        return ret.rstrip()



import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="janch",
    version="0.0.1",
    author="taarimalta",
    author_email="",
    description="A YAML config centric tool to act on your service status",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
        'click >= 7.1.2',
        'PyYAML >= 5.3.1',
        'aiohttp >= 3.7.0',
        'python-dotenv >= 0.14.0'
    ],
    entry_points='''
    [console_scripts]
    janch=janch.cli:main
    '''
)



"""Formatters set the style in which message is logged.
The formatters return a string which can be printed to cli or logged elsewhere

"""
import json
from abc import ABC

from janch.utils.constants import NO_ERROR
from janch.utils.display import FixedWidth


class Formatter(ABC):
    """Abstract base class for a formatter. You can implement a custom formatter by extending
    this class and overriding the methods
    """

    @staticmethod
    def type() -> str:
        """This is the string that identifies the specific formatter

        Returns: str

        """
        raise NotImplementedError("Please specify type for this formatter")

    async def main(self, params: dict) -> str:
        """The main formatting logic goes here.

        The params contain all the information that
        can be displayed by the formatter

        Args:
            params: dict

        Returns:

        """
        raise NotImplementedError("Please specify a method for formatting")

    async def format(self, item, settings, gathered, inspected):
        """This method creates the params for the main function

        Ideally this method should not be overridden by the extending class

        Args:
            item:
            settings:
            gathered:
            inspected:

        Returns:

        """
        params = {
            'item': item,
            'settings': settings,
            'gathered': gathered,
            'inspected': inspected,
            'expecteds': settings['inspect'],
            'actuals': {},
            'matches': {},
            'inspect_count': 0,
            'match_count': 0,
            'match_percent': 0
        }

        for k, v in inspected.items():
            if v is not None:
                params['actuals'][k] = v['actual']
                params['matches'][k] = v['match']

                params['match_count'] += 1 if v['match'] else 0
                params['inspect_count'] += 1

        params['match_percent'] = round(params['match_count'] * 1.0 / params['inspect_count'], 2)

        header, body = await self.main(params)

        return header, body


class JSONFormatter(Formatter):
    """Formats everything as a single JSON"""

    @staticmethod
    def type() -> str:
        """Return 'json'

        Returns: str

        """
        return 'json'

    async def main(self, params):
        """Simply formats the params as JSON using the json library

        Args:
            params: dict

        Returns: None, str

        """
        return None, json.dumps(params)


class SimpleFormatter(Formatter):
    """Easy to read format"""

    DIM = {
        'item': 32,
        'type': 8,
        'field': 16,
        'expected': 32,
        'actual': 32,
        'match': 6,
        'error': 6,

    }

    def __init__(self):
        self.displayer = FixedWidth(SimpleFormatter.DIM)

    @staticmethod
    def type() -> str:
        """Returns 'simple' as the name of this formatter

        Returns: str

        """
        return 'simple'

    async def main(self, params):
        """Format the results in a simple aligned tabular format

        Columns: item, type, field, expected, actual, match, error

        Args:
            params:

        Returns:

        """
        for k, v in params['settings']['inspect'].items():
            row = {
                'item': params['item'],
                'type': params['settings']['gather']['type'],
                'field': k,
                'expected': v['value'] if isinstance(v, dict) else v,
                'actual': params['actuals'].get(k),
                'match': params['matches'].get(k) or False,
                'error': params['gathered']['error'] != NO_ERROR
            }

            self.displayer.add_row(row)

        return self.displayer.get_header(), self.displayer.format()


class PipeDelimited(Formatter):
    """Pipe delimited format"""

    @staticmethod
    def type() -> str:
        """Returns 'pipe' as the name of this formatter

        Returns: str

        """
        return 'pipe'

    async def main(self, params):
        """Displays information in a pipe delimited format

        Args:
            params:

        Returns:

        """
        item = params['item']
        gathered = params['gathered']
        expecteds = params['expecteds']
        actuals = params['actuals']
        matches = params['matches']

        return None, f"{item}" \
            f"|{gathered['error']}" \
            f"|{expecteds or ''}" \
            f"|{actuals or ''}" \
            f"|{matches or ''}"


class FriendlyFormatter(Formatter):
    """Natural language format"""

    @staticmethod
    def type() -> str:
        """Returns 'friendly' as the name of this formatter

        Returns: str

        """
        return 'friendly'

    async def main(self, params):
        """Displays information in a friendly format with the information spanning multiple lines

        Args:
            params: dict

        Returns:

        """
        item = params['item']
        gathered = params['gathered']
        expecteds = params['expecteds']
        actuals = params['actuals']
        match_count = params['match_count']
        inspect_count = params['inspect_count']
        settings = params['settings']

        final = f"In {item},"

        def level(msg, n):
            return '\n' + ' ' * (n * 2) + msg

        final += level("Overall Status:", 1)

        final += level(f"Errors: {gathered['error'] is not None}", 2)
        final += level(f"Matches: {match_count}/{inspect_count}", 2)

        if gathered['error']:
            final += level(
                f"Error occured on attempting to read {item} using {settings['gather']}",
                1)
        else:
            final += level(
                f"We expected the following:", 1)

            for k, v in expecteds.items():
                final += level(
                    k + "=" + str(v), 2
                )

            final += level(
                f"We actually detected the following:", 1)

            for k, v in actuals.items():
                final += level(
                    k + "=" + str(v), 2
                )

        return None, final


def get_default_formatters():
    """Returns all formatters with the formatter type/id as key
    """
    formatters = [JSONFormatter, FriendlyFormatter, PipeDelimited, SimpleFormatter]

    return {f.type(): f for f in formatters}



"""Contains methods to get access to all factory methods that make the janch yml possible"""
from janch.components import *


def get_all_gatherers() -> dict:
    """Get all gatherer classes
    Returns: dict with gatherer type names are keys and gatherer classes as values

    """
    return get_default_gatherers()


def get_all_inspectors() -> dict:
    """Get all inspector classes
    Returns: dict with inspector type names are keys and inspector classes as values

    """
    return get_default_inspectors()


def get_all_loggers():
    """Get all logger classes
    Returns: dict with logger type names are keys and logger classes as values

    """
    return get_default_formatters()


def get_all_formatters():
    """Get all formatter classes
    Returns: dict with formatter type names are keys and formatter classes as values

    """
    return get_default_formatters()



"""Inspectors are classes that compare and check the gathered data
"""
from abc import ABC


class Inspector(ABC):
    """Abstract base class for all Inspector classes

    Inspectors are like comparators

    """

    def __init__(self, settings):
        """Pass the dict form of the inspect section from the config file

        Args:
            settings: dict
        """
        self.settings = settings

    @staticmethod
    def type() -> str:
        """This is the string that identifies the specific inspector

        Returns: str

        """
        raise NotImplementedError("Specify a type for this inspector")

    async def match(self, target, expression) -> bool:
        """This is the method that needs to be overridden to implement an Inspector

        Args:
            target: str The value that needs to be inspected. Gathered by gatherer
            expression: str The value that needs to be compared with. Specified in config

        Returns:

        """
        raise NotImplementedError("Use the parameters to assert that target matches expression")

    async def inspect(self, target, expression):
        """Creates parameters for the match method and creates return dic

        Args:
            target: str The value that needs to be inspected. Gathered by gatherer
            expression: str The value that needs to be compared with. Specified in config

        Returns: dict with 'expected', 'actual' and 'match' as keys

        """
        is_match = await self.match(target, expression)

        return {
            'expected': expression,
            'actual': repr(target).strip("\'\""),
            'match': is_match,
        }


class EqualsInspector(Inspector):
    """Checks a==b"""

    @staticmethod
    def type() -> str:
        """Return 'equals'

        Returns: str

        """
        return 'equals'

    async def match(self, target, check_with) -> bool:
        """Checks if the two parameters and equal using ==

        Args:
            target: str The gathered data
            check_with: str

        Returns: bool

        """
        return target == check_with


class RegexInspector(Inspector):
    """Checks regex match"""

    @staticmethod
    def type() -> str:
        """Return 'regex'

        Returns: str

        """
        return 'regex'

    async def match(self, target, expression) -> bool:
        """Checks if the target matches a given regular expression

        Args:
            target: Gathered data
            expression: specified in YML

        Returns: bool

        """
        import re
        pattern = re.compile(expression)
        matches = pattern.match(target)
        return matches is not None


def get_default_inspectors():
    """Returns all inspectors with the inspector type/id as key

    Returns: dict[str: Inspector]

    """
    inspectors = [EqualsInspector, RegexInspector]

    return {i.type(): i for i in inspectors}
